/** load implement at very first time */
/**
 * global error catch
 */
import './events';
import './loader';

/**
 * @public Jolibox JS SDK Entry
 */
import { getSystemInfoSync, canIUse, login, checkSession, request } from './api';
import { RuntimeSDK } from './sdks/runtime';
import { LifecycleSDK } from './sdks/lifecycle';
import { StorageSDK } from './sdks/storage';
import { JoliboxAds } from './sdks/ads';
import { KeyboardSDK } from './sdks/keyboard';
import { TaskTrackerSDK } from './sdks/task';
import { RouterSDK } from './sdks/router';

declare global {
  interface Window {
    JoliboxSDK: typeof JoliboxSDK; // TODO: code review this @Dengxue
    joliboxsdk: {
      runtime: InstanceType<typeof RuntimeSDK>;
      ads: InstanceType<typeof JoliboxAds>;
      lifecycle: InstanceType<typeof LifecycleSDK>;
      storage: InstanceType<typeof StorageSDK>;
      keyboard: InstanceType<typeof KeyboardSDK>;
      task: InstanceType<typeof TaskTrackerSDK>;
      router: InstanceType<typeof RouterSDK>;
    };
  }
}

export class JoliboxSDK {
  readonly jssdkVersion = '__JOLIBOX_LOCAL_SDK_VERSION__';
  readonly runtime = new RuntimeSDK();
  readonly ads = new JoliboxAds();
  readonly lifecycle = new LifecycleSDK();
  readonly storage = new StorageSDK();
  readonly keyboard = new KeyboardSDK();
  readonly task = new TaskTrackerSDK();
  readonly router = new RouterSDK();

  //global API
  getSystemInfoSync = getSystemInfoSync.bind(this);
  canIUse = canIUse.bind(this);
  login = login.bind(this);
  checkSession = checkSession.bind(this);
  request = request.bind(this);

  constructor() {
    window.joliboxsdk = {
      runtime: this.runtime,
      ads: this.ads,
      lifecycle: this.lifecycle,
      storage: this.storage,
      keyboard: this.keyboard,
      task: this.task,
      router: this.router
    };
  }
}

window.JoliboxSDK = JoliboxSDK;
